package com.xbr.xbr_gaode_location_and_track;

import android.app.Activity;

import androidx.annotation.NonNull;

import com.xbr.xbr_gaode_location_and_track.xbr_gaode_location.GeoFlutterLocationPlugin;
import com.xbr.xbr_gaode_location_and_track.xbr_gaode_location.GeoLocationClientImpl;
import com.xbr.xbr_gaode_location_and_track.xbr_gaode_track.GaodeTrackPlugin;
import com.xbr.xbr_gaode_location_and_track.xbr_gaode_track.GsonUtil;
import com.xbr.xbr_gaode_location_and_track.xbr_gaode_track.MapUtil;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * XbrGaodeLocationAndTrackPlugin
 */
public class XbrGaodeLocationAndTrackPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware, EventChannel.StreamHandler {

    private static final String CHANNEL_METHOD_LOCATION = "xbr_gaode_location_and_track";
    private static final String CHANNEL_STREAM_LOCATION = "amap_flutter_location_stream";

    private Activity activity = null;
    private MethodChannel methodChannel;
    private EventChannel eventChannel;

    private GeoFlutterLocationPlugin geoFlutterLocationPlugin;
    private GaodeTrackPlugin gaodeTrackPlugin;

    public GeoFlutterLocationPlugin getGeoLocationPlugin() {
        if (geoFlutterLocationPlugin == null) {
            geoFlutterLocationPlugin = new GeoFlutterLocationPlugin();
        }
        return geoFlutterLocationPlugin;
    }


    public GaodeTrackPlugin getGaodeTrackPlugin() {
        if (gaodeTrackPlugin == null) gaodeTrackPlugin = new GaodeTrackPlugin();
        return gaodeTrackPlugin;
    }


    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        if (getGeoLocationPlugin().getContext() == null) {
            getGeoLocationPlugin().setContext(binding.getApplicationContext());
        }
        methodChannel = new MethodChannel(binding.getBinaryMessenger(), CHANNEL_METHOD_LOCATION);
        methodChannel.setMethodCallHandler(this);

        eventChannel = new EventChannel(binding.getBinaryMessenger(), CHANNEL_STREAM_LOCATION);
        eventChannel.setStreamHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "updatePrivacyStatement":
                getGeoLocationPlugin().updatePrivacyStatement((Map) call.arguments);
                break;
            case "setApiKey":
                getGeoLocationPlugin().setApiKey((Map) call.arguments);
                break;
            case "setLocationOption":
                getGeoLocationPlugin().setLocationOption((Map) call.arguments);
                break;
            case "startLocation":
                getGeoLocationPlugin().startLocation((Map) call.arguments);
                break;
            case "stopLocation":
                getGeoLocationPlugin().stopLocation((Map) call.arguments);
                break;
            case "destroy":
                getGeoLocationPlugin().destroy((Map) call.arguments);
                break;

            //------------------------------------------------------------------------------------
            case "initGaodeTrackSDK": {
                String serviceId = null;
                Integer gatherInterval = 0, uploadInterval = 0, cacheSize = 0, locationMode = 0;
                if (call.hasArgument("serviceId")) serviceId = call.argument("serviceId");
                if (call.hasArgument("gatherInterval")) gatherInterval = call.argument("gatherInterval");
                if (call.hasArgument("uploadInterval")) uploadInterval = call.argument("uploadInterval");
                if (call.hasArgument("cacheSize")) cacheSize = call.argument("cacheSize");
                if (call.hasArgument("locationMode")) locationMode = call.argument("locationMode");
                getGaodeTrackPlugin().initGaodeTrackSDK(activity, serviceId, gatherInterval, uploadInterval, cacheSize, locationMode);
                result.success(toMapJson(true, "成功"));
                break;
            }
            case "addTerminal": {
                //######## 添加终端
                String terminalName = null;
                if (call.hasArgument("terminalName")) terminalName = call.argument("terminalName");
                if (terminalName == null) result.success(toMapJson(false, "添加终端失败，参数不完整！"));
                getGaodeTrackPlugin().addTerminal(terminalName, toResult(result));
                break;
            }
            case "addTrack": {
                //######## 添加轨迹
                String terminalId = null;
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (terminalId == null) result.success(toMapJson(false, "添加轨迹失败，参数不完整！"));
                getGaodeTrackPlugin().addTrack(terminalId, toResult(result));
                break;
            }
            case "startTrack": {
                //######## 开始轨迹上传
                String terminalId = null, trackId = null;
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (call.hasArgument("trackId")) trackId = call.argument("trackId");
                if (terminalId == null || trackId == null) result.success(toMapJson(false, "开始轨迹失败，参数不完整！"));
                getGaodeTrackPlugin().startTrack(terminalId, trackId, toResult(result));
                break;
            }
            case "startGather":
                //######## 开始采集
                getGaodeTrackPlugin().startGather(toResult(result));
                break;
            case "stopGather":
                //######## 停止采集
                getGaodeTrackPlugin().stopGather(toResult(result));
                break;
            case "stopTrack": {
                //######## 停止轨迹上传
                getGaodeTrackPlugin().stopTrack(toResult(result));
                break;
            }
            case "queryLatestPoint": {
                //######## 查询当前位置
                String serviceId = null, terminalId = null;
                if (call.hasArgument("serviceId")) serviceId = call.argument("serviceId");
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (serviceId == null || terminalId == null) result.success(toMapJson(false, "查询位置失败，参数不完整！"));
                getGaodeTrackPlugin().queryLatestPoint(serviceId, terminalId, toResult(result));
                break;
            }
            case "queryTerminalTrack": {
                //######## 查存轨迹
                String serviceId = null, terminalId = null, trackId = null;
                Long startTime = null, endTime = null;
                Integer noising = null, bindLoad = null, accFilter = null, driveMode = null, disCompensation = null, compensationMeter = null, includePoints = null, page = null, limit = null;
                if (call.hasArgument("serviceId")) serviceId = call.argument("serviceId");
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (call.hasArgument("trackId")) trackId = call.argument("trackId");
                if (call.hasArgument("startTime")) startTime = call.argument("startTime");
                if (call.hasArgument("endTime")) endTime = call.argument("endTime");
                if (call.hasArgument("noising")) noising = call.argument("noising");
                if (call.hasArgument("bindLoad")) bindLoad = call.argument("bindLoad");
                if (call.hasArgument("accFilter")) accFilter = call.argument("accFilter");
                if (call.hasArgument("driveMode")) driveMode = call.argument("driveMode");
                if (call.hasArgument("disCompensation")) disCompensation = call.argument("disCompensation");
                if (call.hasArgument("compensationMeter")) compensationMeter = call.argument("compensationMeter");
                if (call.hasArgument("includePoints")) includePoints = call.argument("includePoints");
                if (call.hasArgument("page")) page = call.argument("page");
                if (call.hasArgument("limit")) limit = call.argument("limit");
                if (serviceId == null || terminalId == null || trackId == null) result.success(toMapJson(false, "查询位置失败，参数不完整！"));
                getGaodeTrackPlugin().queryTerminalTrack(serviceId, terminalId, trackId, startTime, endTime, noising, bindLoad, accFilter, driveMode, disCompensation, compensationMeter, includePoints, page, limit, toResult(result));
                break;
            }
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        methodChannel.setMethodCallHandler(null);
        eventChannel.setStreamHandler(null);
        for (Map.Entry<String, GeoLocationClientImpl> entry : getGeoLocationPlugin().getLocationClientMap().entrySet()) {
            entry.getValue().destroy();
        }
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }

    @Override
    public void onListen(Object arguments, EventChannel.EventSink eventSink) {
        getGeoLocationPlugin().setEventSink(eventSink);
    }

    @Override
    public void onCancel(Object arguments) {
        for (Map.Entry<String, GeoLocationClientImpl> entry : getGeoLocationPlugin().getLocationClientMap().entrySet()) {
            entry.getValue().stopLocation();
        }
    }


    public String toMapJson(Boolean success, String msg) {
        return toMapJson(success ? 0 : -1, success, msg, null);
    }

    public <T> GaodeTrackPlugin.Callback<T> toResult(@NonNull Result result) {
        return (code, success, msg, object) -> result.success(toMapJson(code, success, msg, object));
    }

    public String toMapJson(int code, Boolean success, String msg, Object data) {
        Map<String, Object> json = MapUtil.generate(map -> {
            map.put("code", code);
            map.put("success", success);
            map.put("msg", msg);
            map.put("data", data);
        });
        return GsonUtil.toJson(json);
    }

}
