package com.xbr.xbr_gaode_location_and_track.xbr_gaode_location;

import android.content.Context;
import android.text.TextUtils;

import com.amap.api.location.AMapLocationClient;

import java.lang.reflect.Method;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import io.flutter.plugin.common.EventChannel;

/** 高德地图定位sdkFlutterPlugin */
public class GeoFlutterLocationPlugin {
  private Map<String, GeoLocationClientImpl> locationClientMap = new ConcurrentHashMap<>(8);

  public Map<String, GeoLocationClientImpl> getLocationClientMap() {
    return locationClientMap;
  }
  private Context mContext = null;
  private EventChannel.EventSink mEventSink = null;


  public void setContext(Context mContext) {
    this.mContext = mContext;
  }

  public Context getContext() {
    return mContext;
  }

  public void setEventSink(EventChannel.EventSink mEventSink) {
    this.mEventSink = mEventSink;
  }

  /**
   * 开始定位
   */
  public void startLocation(Map argsMap) {
    GeoLocationClientImpl locationClientImp = getLocationClientImp(argsMap);
    if (null != locationClientImp) {
      locationClientImp.startLocation();
    }
  }


  /**
   * 停止定位
   */
  public void stopLocation(Map argsMap) {
    GeoLocationClientImpl locationClientImp = getLocationClientImp(argsMap);
    if (null != locationClientImp) {
      locationClientImp.stopLocation();
    }
  }

  /**
   * 销毁
   *
   * @param argsMap
   */
  public void destroy(Map argsMap) {
    GeoLocationClientImpl locationClientImp = getLocationClientImp(argsMap);
    if (null != locationClientImp) {
      locationClientImp.destroy();
      locationClientMap.remove(getPluginKeyFromArgs(argsMap));
    }
  }


  /**
   * 设置apikey
   *
   * @param apiKeyMap
   */
  public void setApiKey(Map apiKeyMap) {
    if (null != apiKeyMap) {
      if (apiKeyMap.containsKey("android")
              && !TextUtils.isEmpty((String) apiKeyMap.get("android"))) {
        AMapLocationClient.setApiKey((String) apiKeyMap.get("android"));
      }
    }
  }

  public void updatePrivacyStatement(Map privacyShowMap) {
    if (null != privacyShowMap) {
      Class<AMapLocationClient> locationClazz = AMapLocationClient.class;

      if (privacyShowMap.containsKey("hasContains") && privacyShowMap.containsKey("hasShow")) {
        boolean hasContains = (boolean) privacyShowMap.get("hasContains");
        boolean hasShow = (boolean) privacyShowMap.get("hasShow");
        try {
          Method showMethod = locationClazz.getMethod("updatePrivacyShow", Context.class, boolean.class, boolean.class);;
          showMethod.invoke(null, mContext, hasContains, hasShow);
        } catch (Throwable e) {
//          e.printStackTrace();
        }
      }

      if (privacyShowMap.containsKey("hasAgree")) {
        boolean hasAgree = (boolean) privacyShowMap.get("hasAgree");
        try {
          Method agreeMethod = locationClazz.getMethod("updatePrivacyAgree", Context.class, boolean.class);
          agreeMethod.invoke(null, mContext, hasAgree);
        } catch (Throwable e) {
//            e.printStackTrace();
        }
      }
    }
  }

  /**
   * 设置定位参数
   *
   * @param argsMap
   */
  public void setLocationOption(Map argsMap) {
    GeoLocationClientImpl locationClientImp = getLocationClientImp(argsMap);
    if (null != locationClientImp) {
      locationClientImp.setLocationOption(argsMap);
    }
  }


  private GeoLocationClientImpl getLocationClientImp(Map argsMap) {
    if (null == locationClientMap) {
      locationClientMap = new ConcurrentHashMap<String, GeoLocationClientImpl>(8);
    }

    String pluginKey = getPluginKeyFromArgs(argsMap);
    if (TextUtils.isEmpty(pluginKey)) {
      return null;
    }

    if (!locationClientMap.containsKey(pluginKey)) {
      GeoLocationClientImpl locationClientImp = new GeoLocationClientImpl(mContext, pluginKey, mEventSink);
      locationClientMap.put(pluginKey, locationClientImp);
    }
    return locationClientMap.get(pluginKey);
  }

  private String getPluginKeyFromArgs(Map argsMap) {
    String pluginKey = null;
    try {
      if (null != argsMap) {
        pluginKey = (String) argsMap.get("pluginKey");
      }
    } catch (Throwable e) {
      e.printStackTrace();
    }
    return pluginKey;
  }
}
