package com.xbr.xbr_gaode_location_and_track.xbr_gaode_track;

import android.app.Activity;

import com.amap.api.track.AMapTrackClient;
import com.amap.api.track.OnTrackLifecycleListener;
import com.amap.api.track.query.entity.DriveMode;
import com.amap.api.track.query.entity.LocationMode;
import com.amap.api.track.query.entity.Point;
import com.amap.api.track.query.entity.Track;

import java.util.List;


public class GaodeTrackPlugin implements OnTrackLifecycleListener {

    public interface Callback<T>{
        void compete(int code,boolean success,String msg,T object);
    }

    private AMapTrackClient aMapTrackClient;
    private Activity activity;
    private QueryClient terminalClient;
    private TrackClient trackClient;
    private String serviceId; //初始化时保存
    private String terminalId; //轨迹上报时保存
    private String trackId; //轨迹上报时保存

    public AMapTrackClient getaMapTrackClient() {
        if (activity == null) return null;
        if (aMapTrackClient == null) aMapTrackClient = new AMapTrackClient(activity.getApplicationContext());
        return aMapTrackClient;
    }

    public QueryClient getTerminalClient() {
        if (aMapTrackClient == null) return null;
        if (terminalClient == null) terminalClient = new QueryClient(aMapTrackClient);
        return terminalClient;
    }

    public TrackClient getTrackClient() {
        if (aMapTrackClient == null) return null;
        if (trackClient == null) trackClient = new TrackClient(aMapTrackClient);
        return trackClient;
    }

    // 初始化猎鹰SDK
    public void initGaodeTrackSDK(Activity activity,String serviceId, Integer gatherInterval, Integer uploadInterval, Integer cacheSize, Integer locationMode) {
        this.activity = activity;
        this.serviceId = serviceId;
        if (gatherInterval == null) gatherInterval = 5;
        if (uploadInterval == null) uploadInterval = 50;
        if (cacheSize == null) cacheSize = 20;
        if (locationMode == null) locationMode = LocationMode.HIGHT_ACCURACY;
        ///------------------------------------------
        getaMapTrackClient().setInterval(gatherInterval, uploadInterval);//采集间隔
        getaMapTrackClient().setCacheSize(cacheSize);//最大缓存
        getaMapTrackClient().setLocationMode(locationMode);
        getaMapTrackClient().setOnTrackListener(this);
    }

    ///// 创建终端
    public void addTerminal(String terminalName, Callback<String> callback) {
        getTerminalClient().addTerminal(serviceId, terminalName, callback);
    }

    ///// 创建轨迹
    public void addTrack(String terminalId, Callback<String> callback) {
        getTerminalClient().addTrack(serviceId, terminalId, callback);
    }

    ///// 轨迹上报
    public void startTrack(String terminalId, String trackId, Callback<String> callback) {
        this.terminalId = terminalId;
        this.trackId = trackId;
        getTrackClient().startTrack(activity, serviceId, terminalId, trackId, callback);
    }

    ///// 开始采集
    public void startGather(Callback<String> callback) {
        getTrackClient().startGather(callback);
    }

    ///// 停止采集
    public void stopGather(Callback<String> callback) {
        getTrackClient().stopGather(callback);
    }

    ///// 停止上报
    public void stopTrack(Callback<String> callback) {
        getTrackClient().stopTrack(serviceId, terminalId, trackId, callback);
    }

    //// 获取终端位置
    public void queryLatestPoint(String serviceId, String terminalId,Callback<Point> callback) {
        getTerminalClient().queryLatestPoint(serviceId, terminalId, callback);
    }

    //// 获取指定轨迹
    public void queryTerminalTrack(String serviceId, String terminalId, String trackId, Long startTime, Long endTime,
                                   Integer noising, Integer bindLoad, Integer accFilter, Integer driveMode,
                                   Integer disCompensation, Integer compensationMeter, Integer includePoints, Integer page, Integer limit,
                                   Callback<List<Track>> callback) {
        if (startTime == null) startTime = System.currentTimeMillis() - 12 * 60 * 60 * 1000;
        if (endTime == null)endTime = System.currentTimeMillis();
        if (noising == null) noising = 0;
        if (bindLoad == null) bindLoad = 1;
        if (accFilter == null) accFilter = 0;
        if (driveMode == null) driveMode = DriveMode.DRIVING;
        if (disCompensation == null) disCompensation = 1;
        if (compensationMeter == null) compensationMeter = 2000;
        if (includePoints == null) includePoints = 1;
        if (page == null) page = 1;
        if (limit == null) limit = 100;
        getTerminalClient().queryTerminalTrack(serviceId, terminalId, trackId, startTime, endTime, noising, bindLoad,
            accFilter, driveMode, disCompensation, compensationMeter, includePoints, page, limit, callback);
    }

    /// 绑定服务回调
    @Override
    public void onBindServiceCallback(int i, String s) {

    }

    @Override
    public void onStartTrackCallback(int status, String msg) {
    }

    @Override
    public void onStartGatherCallback(int status, String msg) {
    }

    @Override
    public void onStopGatherCallback(int i, String s) {
    }

    @Override
    public void onStopTrackCallback(int i, String s) {
    }

}
