package com.xbr.xbr_gaode_location_and_track.xbr_gaode_track;

import com.amap.api.track.AMapTrackClient;
import com.amap.api.track.query.entity.Point;
import com.amap.api.track.query.entity.Track;
import com.amap.api.track.query.model.AddTerminalRequest;
import com.amap.api.track.query.model.AddTerminalResponse;
import com.amap.api.track.query.model.AddTrackRequest;
import com.amap.api.track.query.model.AddTrackResponse;
import com.amap.api.track.query.model.DistanceResponse;
import com.amap.api.track.query.model.HistoryTrackResponse;
import com.amap.api.track.query.model.LatestPointRequest;
import com.amap.api.track.query.model.LatestPointResponse;
import com.amap.api.track.query.model.OnTrackListener;
import com.amap.api.track.query.model.ParamErrorResponse;
import com.amap.api.track.query.model.QueryTerminalResponse;
import com.amap.api.track.query.model.QueryTrackRequest;
import com.amap.api.track.query.model.QueryTrackResponse;

import java.util.List;

public class QueryClient implements OnTrackListener {

    private final AMapTrackClient aMapTrackClient;
    private GaodeTrackPlugin.Callback<String> createTerminalCallback;
    private GaodeTrackPlugin.Callback<String> createTrackCallback;
    private GaodeTrackPlugin.Callback<Point> queryLatestPointCallback;
    private GaodeTrackPlugin.Callback<List<Track>> queryTrackListCallback;

    public QueryClient(AMapTrackClient aMapTrackClient) {
        this.aMapTrackClient = aMapTrackClient;
    }

    ///// 创建终端
    public void addTerminal(String serviceId, String terminalName, GaodeTrackPlugin.Callback<String> callback) {
        this.createTerminalCallback = callback;
        aMapTrackClient.addTerminal(new AddTerminalRequest(terminalName, Long.parseLong(serviceId)), this);
    }

    ///// 创建轨迹
    public void addTrack(String serviceId, String terminalId, GaodeTrackPlugin.Callback<String> callback) {
        this.createTrackCallback = callback;
        aMapTrackClient.addTrack(new AddTrackRequest(Long.parseLong(serviceId), Long.parseLong(terminalId)), this);
    }

    //// 获取终端位置
    public void queryLatestPoint(String serviceId, String terminalId, GaodeTrackPlugin.Callback<Point> callback) {
        this.queryLatestPointCallback = callback;
        aMapTrackClient.queryLatestPoint(new LatestPointRequest(Long.parseLong(serviceId), Long.parseLong(terminalId)), this);
    }

    //// 获取指定轨迹
    public void queryTerminalTrack(String serviceId, String terminalId, String trackId,
                                   Long startTime, Long endTime,
                                   int noising,//0
                                   int bindLoad,//1
                                   int accFilter,//0
                                   int driveMode,//DriveMode.DRIVING
                                   int disCompensation,//1
                                   int compensationMeter,//2000,
                                   int includePoints,//1
                                   int page, int limit, // 1- 100
                                   GaodeTrackPlugin.Callback<List<Track>> callback) {
        this.queryTrackListCallback = callback;
        QueryTrackRequest queryTrackRequest = new QueryTrackRequest(
            Long.parseLong(serviceId),
            Long.parseLong(terminalId),
            Long.parseLong(trackId),    // 轨迹id，传-1表示查询所有轨迹
            startTime,
            endTime,
            noising,      // 不启用去噪
            bindLoad,   // 绑路
            accFilter,      // 不进行精度过滤
            driveMode,  // 当前仅支持驾车模式
            disCompensation,     // 距离补偿
            compensationMeter,   // 距离补偿，只有超过2km的点才启用距离补偿
            includePoints,  // 结果应该包含轨迹点信息
            page,  // 返回第1页数据，由于未指定轨迹，分页将失效
            limit    // 一页不超过100条
        );
        aMapTrackClient.queryTerminalTrack(queryTrackRequest, this);
    }

    @Override
    public void onCreateTerminalCallback(AddTerminalResponse response) {
        if (this.createTerminalCallback == null) return;
        if (response.isSuccess()) {
            long terminalId = response.getTid();
            this.createTerminalCallback.compete(0,true,"成功",String.valueOf(terminalId));
        }else{
            this.createTerminalCallback.compete(response.getErrorCode(),false,response.getErrorMsg(),null);
        }
    }

    @Override
    public void onAddTrackCallback(AddTrackResponse response) {
        if (this.createTrackCallback == null) return;
        if (response.isSuccess()) {
            long trackId = response.getTrid();
            this.createTrackCallback.compete(0,true,"成功",String.valueOf(trackId));
        }else{
            this.createTrackCallback.compete(response.getErrorCode(),false,response.getErrorMsg(),null);
        }
    }

    @Override
    public void onLatestPointCallback(LatestPointResponse response) {
        if (this.queryLatestPointCallback == null) return;
        if (response.isSuccess()) {
            Point point = response.getLatestPoint().getPoint();
            this.queryLatestPointCallback.compete(0,true,"成功",point);
        }else{
            this.queryLatestPointCallback.compete(response.getErrorCode(),false,response.getErrorMsg(),null);
        }
    }

    @Override
    public void onQueryTrackCallback(QueryTrackResponse response) {
        if (this.queryTrackListCallback == null) return;
        if (response.isSuccess()) {
            List<Track> tracks = response.getTracks();
            this.queryTrackListCallback.compete(0,true,"成功",tracks);
        }else{
            this.queryTrackListCallback.compete(response.getErrorCode(),false,response.getErrorMsg(),null);
        }
    }

    @Override
    public void onQueryTerminalCallback(QueryTerminalResponse queryTerminalResponse) {

    }


    @Override
    public void onDistanceCallback(DistanceResponse distanceResponse) {

    }


    @Override
    public void onHistoryTrackCallback(HistoryTrackResponse historyTrackResponse) {

    }

    @Override
    public void onParamErrorCallback(ParamErrorResponse paramErrorResponse) {

    }
}
