package com.xbr.xbr_gaode_location_and_track.xbr_gaode_track;

import static android.content.Context.NOTIFICATION_SERVICE;
import android.app.Activity;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Intent;
import android.os.Build;

import com.amap.api.track.AMapTrackClient;
import com.amap.api.track.ErrorCode;
import com.amap.api.track.OnTrackLifecycleListener;
import com.amap.api.track.TrackParam;
import com.xbr.xbr_gaode_location_and_track.R;

public class TrackClient implements OnTrackLifecycleListener {
    private static final String CHANNEL_ID_SERVICE_RUNNING = "CHANNEL_ID_SERVICE_RUNNING";

    private final AMapTrackClient aMapTrackClient;
    private GaodeTrackPlugin.Callback<String>  startTrackCallback;
    private GaodeTrackPlugin.Callback<String>  startGatherCallback;
    private GaodeTrackPlugin.Callback<String>  stopGatherCallback;
    private GaodeTrackPlugin.Callback<String>  stopTrackCallback;

    public TrackClient(AMapTrackClient aMapTrackClient) {
        this.aMapTrackClient = aMapTrackClient;
    }

    // 创建前台服务通知
    private Notification createNotification(Activity activity) {
        Notification.Builder builder;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationManager nm = (NotificationManager) activity.getSystemService(NOTIFICATION_SERVICE);
            NotificationChannel channel = new NotificationChannel(CHANNEL_ID_SERVICE_RUNNING, "app service", NotificationManager.IMPORTANCE_LOW);
            nm.createNotificationChannel(channel);
            builder = new Notification.Builder(activity.getApplicationContext(), CHANNEL_ID_SERVICE_RUNNING);
        } else {
            builder = new Notification.Builder(activity.getApplicationContext());
        }
        Intent nfIntent = new Intent(activity, activity.getClass());
        nfIntent.setFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP | Intent.FLAG_ACTIVITY_CLEAR_TOP);
        builder.setContentIntent(PendingIntent.getActivity(activity,0, nfIntent, 0))
            .setSmallIcon(R.mipmap.lieyin)
            .setContentTitle("猎鹰轨迹采集中")
            .setContentText("正在使用猎鹰轨迹SDK");
        return builder.build();
    }

    // 指定轨迹上报 轨迹等的创建全部在后端完成
    public void startTrack(Activity activity, String serviceId, String terminalId, String trackId, GaodeTrackPlugin.Callback<String> callback){
        this.startTrackCallback = callback;
        TrackParam trackParam = new TrackParam(Long.parseLong(serviceId), Long.parseLong(terminalId));
        trackParam.setTrackId(Long.parseLong(trackId));
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            trackParam.setNotification(createNotification(activity));
        }
        aMapTrackClient.startTrack(trackParam, this);
    }

    /// 开始采集
    public void startGather(GaodeTrackPlugin.Callback<String> callback){
        this.startGatherCallback = callback;
        aMapTrackClient.startGather(this);
    }

    /// 停止采集
    public void stopGather(GaodeTrackPlugin.Callback<String> callback){
        this.stopGatherCallback = callback;
        aMapTrackClient.stopGather(this);
    }

    // 停止轨迹上报
    public void stopTrack(String serviceId, String terminalId, String trackId, GaodeTrackPlugin.Callback<String> callback){
        this.stopTrackCallback = callback;
        TrackParam trackParam = new TrackParam(Long.parseLong(serviceId), Long.parseLong(terminalId));
        trackParam.setTrackId(Long.parseLong(trackId));
        aMapTrackClient.stopTrack(trackParam, this);
    }

    /// 开启轨迹回调
    @Override
    public void onStartTrackCallback(int status, String msg) {
        if(startTrackCallback==null) return;
        if (status == ErrorCode.TrackListen.START_TRACK_SUCEE ||
            status == ErrorCode.TrackListen.START_TRACK_SUCEE_NO_NETWORK ||
            status == ErrorCode.TrackListen.START_TRACK_ALREADY_STARTED) {
            startTrackCallback.compete(status,true,msg,null);
        }else{
            startTrackCallback.compete(status,false,msg,null);
        }
    }

    /// 定位采集开启回调
    @Override
    public void onStartGatherCallback(int status, String msg) {
        if(startGatherCallback==null) return;
        if (status == ErrorCode.TrackListen.START_GATHER_SUCEE ||
            status == ErrorCode.TrackListen.START_GATHER_ALREADY_STARTED) {
            startGatherCallback.compete(status,true,msg,null);
        }else{
            startTrackCallback.compete(status,false,msg,null);
        }
    }

    /// 定位采集停止回调
    @Override
    public void onStopGatherCallback(int status, String msg) {
        if(stopGatherCallback==null) return;
        if (status == ErrorCode.TrackListen.STOP_GATHER_SUCCE) {
            stopGatherCallback.compete(status,true,msg,null);
        }else{
            startTrackCallback.compete(status,false,msg,null);
        }
    }

    /// 停止轨迹上报回调
    @Override
    public void onStopTrackCallback(int status, String msg) {
        if(stopTrackCallback==null) return;
        if (status == ErrorCode.TrackListen.STOP_TRACK_SUCCE) {
            stopTrackCallback.compete(status,true,msg,null);
        }else{
            startTrackCallback.compete(status,false,msg,null);
        }
    }

    @Override
    public void onBindServiceCallback(int i, String s) {

    }


}
